@@ -80,7 +80,7 @@ func TestCaptureMessageEmptyString(t *testing.T) {
 	}
 	got := transport.lastEvent
 	opts := cmp.Options{
-		cmpopts.IgnoreFields(Event{}, "sdkMetaData", "attachments"),
+		cmpopts.IgnoreFields(Event{}, "sdkMetaData"),
 		cmp.Transformer("SimplifiedEvent", func(e *Event) *Event {
 			return &Event{
 				Exception: e.Exception,
@@ -164,14 +164,20 @@ func TestCaptureException(t *testing.T) {
 				{
 					Type:  "*sentry.customErr",
 					Value: "wat",
-					// No Stacktrace, because we can't tell where the error came
-					// from and because we have a stack trace in the most recent
-					// error in the chain.
+					Mechanism: &Mechanism{
+						ExceptionID:      0,
+						IsExceptionGroup: true,
+					},
 				},
 				{
 					Type:       "*errors.withStack",
 					Value:      "wat",
 					Stacktrace: &Stacktrace{Frames: []Frame{}},
+					Mechanism: &Mechanism{
+						ExceptionID:      1,
+						ParentID:         Pointer(0),
+						IsExceptionGroup: true,
+					},
 				},
 			},
 		},
@@ -193,11 +199,20 @@ func TestCaptureException(t *testing.T) {
 				{
 					Type:  "*sentry.customErr",
 					Value: "wat",
+					Mechanism: &Mechanism{
+						ExceptionID:      0,
+						IsExceptionGroup: true,
+					},
 				},
 				{
 					Type:       "*sentry.customErrWithCause",
 					Value:      "err",
 					Stacktrace: &Stacktrace{Frames: []Frame{}},
+					Mechanism: &Mechanism{
+						ExceptionID:      1,
+						ParentID:         Pointer(0),
+						IsExceptionGroup: true,
+					},
 				},
 			},
 		},
@@ -208,11 +223,20 @@ func TestCaptureException(t *testing.T) {
 				{
 					Type:  "*errors.errorString",
 					Value: "original",
+					Mechanism: &Mechanism{
+						ExceptionID:      0,
+						IsExceptionGroup: true,
+					},
 				},
 				{
 					Type:       "sentry.wrappedError",
 					Value:      "wrapped: original",
 					Stacktrace: &Stacktrace{Frames: []Frame{}},
+					Mechanism: &Mechanism{
+						ExceptionID:      1,
+						ParentID:         Pointer(0),
+						IsExceptionGroup: true,
+					},
 				},
 			},
 		},
@@ -287,7 +311,7 @@ func TestCaptureEvent(t *testing.T) {
 		},
 	}
 	got := transport.lastEvent
-	opts := cmp.Options{cmpopts.IgnoreFields(Event{}, "Release", "sdkMetaData", "attachments")}
+	opts := cmp.Options{cmpopts.IgnoreFields(Event{}, "Release", "sdkMetaData")}
 	if diff := cmp.Diff(want, got, opts); diff != "" {
 		t.Errorf("Event mismatch (-want +got):\n%s", diff)
 	}
@@ -315,7 +339,7 @@ func TestCaptureEventNil(t *testing.T) {
 	}
 	got := transport.lastEvent
 	opts := cmp.Options{
-		cmpopts.IgnoreFields(Event{}, "sdkMetaData", "attachments"),
+		cmpopts.IgnoreFields(Event{}, "sdkMetaData"),
 		cmp.Transformer("SimplifiedEvent", func(e *Event) *Event {
 			return &Event{
 				Exception: e.Exception,
@@ -359,10 +383,12 @@ func TestCaptureCheckIn(t *testing.T) {
 				Duration:    time.Second * 10,
 			},
 			monitorConfig: &MonitorConfig{
-				Schedule:      IntervalSchedule(1, MonitorScheduleUnitHour),
-				CheckInMargin: 10,
-				MaxRuntime:    5000,
-				Timezone:      "Asia/Singapore",
+				Schedule:              IntervalSchedule(1, MonitorScheduleUnitHour),
+				CheckInMargin:         10,
+				MaxRuntime:            5000,
+				Timezone:              "Asia/Singapore",
+				FailureIssueThreshold: 5,
+				RecoveryThreshold:     10,
 			},
 		},
 		{
@@ -374,10 +400,12 @@ func TestCaptureCheckIn(t *testing.T) {
 				Duration:    time.Second * 10,
 			},
 			monitorConfig: &MonitorConfig{
-				Schedule:      CrontabSchedule("40 * * * *"),
-				CheckInMargin: 10,
-				MaxRuntime:    5000,
-				Timezone:      "Asia/Singapore",
+				Schedule:              CrontabSchedule("40 * * * *"),
+				CheckInMargin:         10,
+				MaxRuntime:            5000,
+				Timezone:              "Asia/Singapore",
+				FailureIssueThreshold: 5,
+				RecoveryThreshold:     10,
 			},
 		},
 	}
@@ -552,6 +580,117 @@ func TestBeforeSendTransactionIsCalled(t *testing.T) {
 	assertEqual(t, lastEvent.Contexts["trace"]["span_id"], transaction.SpanID)
 }
 
+func TestIgnoreErrors(t *testing.T) {
+	tests := map[string]struct {
+		ignoreErrors []string
+		message      string
+		expectDrop   bool
+	}{
+		"No Match": {
+			message:      "Foo",
+			ignoreErrors: []string{"Bar", "Baz"},
+			expectDrop:   false,
+		},
+		"Partial Match": {
+			message:      "FooBar",
+			ignoreErrors: []string{"Foo", "Baz"},
+			expectDrop:   true,
+		},
+		"Exact Match": {
+			message:      "Foo Bar",
+			ignoreErrors: []string{"\\bFoo\\b", "Baz"},
+			expectDrop:   true,
+		},
+		"Wildcard Match": {
+			message:      "Foo",
+			ignoreErrors: []string{"F*", "Bar"},
+			expectDrop:   true,
+		},
+		"Match string but not pattern": {
+			message:      "(Foo)",
+			ignoreErrors: []string{"(Foo)"},
+			expectDrop:   true,
+		},
+	}
+
+	for name, tt := range tests {
+		t.Run(name, func(t *testing.T) {
+			scope := &ScopeMock{}
+			transport := &TransportMock{}
+			client, err := NewClient(ClientOptions{
+				Transport:    transport,
+				IgnoreErrors: tt.ignoreErrors,
+			})
+			if err != nil {
+				t.Fatal(err)
+			}
+
+			client.CaptureMessage(tt.message, nil, scope)
+
+			dropped := transport.lastEvent == nil
+			if !(tt.expectDrop == dropped) {
+				t.Errorf("expected event to be dropped")
+			}
+		})
+	}
+}
+
+func TestIgnoreTransactions(t *testing.T) {
+	tests := map[string]struct {
+		ignoreTransactions []string
+		transaction        string
+		expectDrop         bool
+	}{
+		"No Match": {
+			transaction:        "Foo",
+			ignoreTransactions: []string{"Bar", "Baz"},
+			expectDrop:         false,
+		},
+		"Partial Match": {
+			transaction:        "FooBar",
+			ignoreTransactions: []string{"Foo", "Baz"},
+			expectDrop:         true,
+		},
+		"Exact Match": {
+			transaction:        "Foo Bar",
+			ignoreTransactions: []string{"\\bFoo\\b", "Baz"},
+			expectDrop:         true,
+		},
+		"Wildcard Match": {
+			transaction:        "Foo",
+			ignoreTransactions: []string{"F*", "Bar"},
+			expectDrop:         true,
+		},
+		"Match string but not pattern": {
+			transaction:        "(Foo)",
+			ignoreTransactions: []string{"(Foo)"},
+			expectDrop:         true,
+		},
+	}
+
+	for name, tt := range tests {
+		t.Run(name, func(t *testing.T) {
+			transport := &TransportMock{}
+			ctx := NewTestContext(ClientOptions{
+				EnableTracing:      true,
+				TracesSampleRate:   1.0,
+				Transport:          transport,
+				IgnoreTransactions: tt.ignoreTransactions,
+			})
+
+			transaction := StartTransaction(ctx,
+				tt.transaction,
+			)
+			transaction.Finish()
+
+			dropped := transport.lastEvent == nil
+			if !(tt.expectDrop == dropped) {
+				t.Errorf("expected event to be dropped")
+			}
+		})
+	}
+}
+
 func TestSampleRate(t *testing.T) {
 	tests := []struct {
 		SampleRate float64
@@ -654,7 +793,7 @@ func TestRecover(t *testing.T) {
 		}
 		got := events[0]
 		opts := cmp.Options{
-			cmpopts.IgnoreFields(Event{}, "sdkMetaData", "attachments"),
+			cmpopts.IgnoreFields(Event{}, "sdkMetaData"),
 			cmp.Transformer("SimplifiedEvent", func(e *Event) *Event {
 				return &Event{
 					Message:   e.Message,
