@@ -6,15 +6,7 @@ import (
 )
 
 // The version of the SDK.
-// Deprecated: To be removed in 0.25.0. Use SDKVersion instead.
-const Version = SDKVersion
-
-// The version of the SDK.
-const SDKVersion = "0.24.1"
-
-// The identifier of the SDK.
-// Deprecated: To be removed in 0.25.0. Use Client.GetSDKIdentifier() instead.
-const SDKIdentifier = "sentry.go"
+const SDKVersion = "0.29.1"
 
 // apiVersion is the minimum version of the Sentry API compatible with the
 // sentry-go SDK.
@@ -80,18 +72,17 @@ func Recover() *EventID {
 
 // RecoverWithContext captures a panic and passes relevant context object.
 func RecoverWithContext(ctx context.Context) *EventID {
-	if err := recover(); err != nil {
-		var hub *Hub
-
-		if HasHubOnContext(ctx) {
-			hub = GetHubFromContext(ctx)
-		} else {
-			hub = CurrentHub()
-		}
-
-		return hub.RecoverWithContext(ctx, err)
+	err := recover()
+	if err == nil {
+		return nil
 	}
-	return nil
+
+	hub := GetHubFromContext(ctx)
+	if hub == nil {
+		hub = CurrentHub()
+	}
+
+	return hub.RecoverWithContext(ctx, err)
 }
 
 // WithScope is a shorthand for CurrentHub().WithScope.
